<?php

namespace Drupal\photos;

use Drupal\Core\Url;

class PhotosAlbum {

  /**
   * Album ID {node}.nid.
   */
  protected $pid;

  /**
   * Constructs a PhotosAlbum object.
   *
   * @param $nid
   *   Album ID {node}.nid.
   */
  public function __construct($nid) {
    $this->pid = $nid;
  }

  /**
   * Page and Teaser display settings.
   */
  public function nodeView($node, $display, $view_mode) {
    $album = array();
    $default_style = 'medium';
    if ($display <> 0) {
      $default_order = \Drupal::config('photos.settings')->get('photos_display_imageorder');
      $order = explode('|', (isset($node->album['imageorder']) ? $node->album['imageorder'] : $default_order));
      $order = \Drupal\photos\PhotosAlbum::_photos_order_value_change($order[0], $order[1]);
      $default_style = \Drupal::config('photos.settings')->get('photos_display_' . $view_mode . '_imagesize') ?: 'thumbnail';
      $style_name = isset($node->album[$view_mode . '_imagesize']) ? $node->album[$view_mode . '_imagesize'] : $default_style;
    }
    switch ($display) {
      case 0:
        // Display none.
      break;
      case 1:
        // Display cover.
        $album = array();
        if (isset($node->album['cover'])) {
          if (isset($node->album['cover']['view'])) {
            $album = $node->album['cover']['view'];
          }
          else {
            if (isset($node->album['cover']['uri'])) {
              $image = new \stdClass();
              $variables = array(
                'style_name' => $style_name,
                'uri' => $node->album['cover']['uri'],
              );

              // The image.factory service will check if our image is valid.
              $image_info = \Drupal::service('image.factory')->get($node->album['cover']['uri']);
              if ($image_info->isValid()) {
                $variables['width'] = $image_info->getWidth();
                $variables['height'] = $image_info->getHeight();
              }
              else {
                $variables['width'] = $variables['height'] = NULL;
              }

              $image_render_array = [
                '#theme' => 'image_style',
                '#uri' => $node->album['cover']['uri'],
                '#style_name' => $style_name,
              ];
              $image->view = $image_render_array;
              $image->href = 'photos/album/' . $node->id();
              $image->uri = $node->album['cover']['uri'];
              $image->pid = $node->id();
              $image->title = $node->getTitle();

              $image_render_array = array(
                '#theme' => 'photos_image_html',
                '#image' => $image,
                '#style_name' => $style_name,
              );
              $album = $image_render_array;
            }
          }
        }
      break;
      case 2:
        // Display thumbnails.
        $column = (isset($_GET['field'])) ? \Drupal\Component\Utility\Html::escape($_GET['field']) : 0;
        $sort = (isset($_GET['sort'])) ? \Drupal\Component\Utility\Html::escape($_GET['sort']) : 0;
        $view_num = \Drupal::config('photos.settings')->get('photos_display_' . $view_mode . '_viewnum') ?: 10;
        $limit = isset($node->album[$view_mode . '_viewnum']) ? $node->album[$view_mode . '_viewnum'] : $view_num;

        $term = \Drupal\photos\PhotosAlbum::_photos_order_value($column, $sort, $limit, $order);
        $query = db_select('file_managed', 'f');
        $query->join('photos_image', 'p', 'p.fid = f.fid');
        $query->fields('f', array('fid'));
        $query->condition('p.pid', $node->id());
        $query->orderBy($term['order']['column'], $term['order']['sort']);
        $query->range(0, $term['limit']);
        $result = $query->execute();

        $i = 0;
        // Necessary when upgrading from D6 to D7.
        $image_styles = image_style_options(FALSE);
        if (!isset($image_styles[$style_name])) {
          $style_name = \Drupal::config('photos.settings')->get('photos_display_teaser_imagesize');
        }
        $album = array();
        // Thumbnails.
        foreach ($result as $data) {
          $photos_image = new \Drupal\photos\PhotosImage($data->fid);
          $variables = array(
            'href' => 'photos/image/' . $data->fid
          );
          $album[] = $photos_image->view($style_name, $variables);
          ++$i;
        }
      break;
      case 3:
        // Get cover.
        $cover = FALSE;
        if (isset($node->album['cover']) && isset($node->album['cover']['uri'])) {
          $image_render_array = array(
            '#theme' => 'image_style',
            '#style_name' => $style_name,
            '#uri' => $node->album['cover']['uri'],
            '#title' => $node->getTitle(),
            '#alt' => $node->getTitle()
          );
          $cover = $image_render_array;
        }

        if ($cover) {
          // Cover with colorbox gallery.
          $column = (isset($_GET['field'])) ? \Drupal\Component\Utility\Html::escape($_GET['field']) : 0;
          $sort = (isset($_GET['sort'])) ? \Drupal\Component\Utility\Html::escape($_GET['sort']) : 0;
          $view_num = \Drupal::config('photos.settings')->get('photos_display_' . $view_mode . '_viewnum') ?: 10;
          $limit = FALSE;

          // Query all images in gallery.
          $term = \Drupal\photos\PhotosAlbum::_photos_order_value($column, $sort, $limit, $order);
          $query = db_select('file_managed', 'f');
          $query->join('photos_image', 'p', 'p.fid = f.fid');
          $query->join('users_field_data', 'ufd', 'ufd.uid = f.uid');
          $query->fields('f', array('uri', 'filemime', 'created', 'filename', 'filesize'))
            ->fields('p')
            ->fields('ufd', array('uid', 'name'));
          $query->condition('p.pid', $node->id());
          $query->orderBy($term['order']['column'], $term['order']['sort']);
          $result = $query->execute();

          $i = 0;
          // Setup colorbox.
          if (\Drupal::moduleHandler()->moduleExists('colorbox')) {
            $style = \Drupal::config('colorbox.settings')->get('custom.style');
            $album['#attached']['library'] = array('colorbox/colorbox', "colorbox/$style");
            $colorbox_height = \Drupal::config('photos.settings')->get('photos_display_colorbox_max_height') ?: 100;
            $colorbox_width = \Drupal::config('photos.settings')->get('photos_display_colorbox_max_width') ?: 50;
            $js_settings = array(
              'maxWidth' => $colorbox_width . '%',
              'maxHeight' => $colorbox_height . '%'
            );
            $album['#attached']['drupalSettings']['colorbox'] = $js_settings;
          }
          // Display cover and list colorbox image links.
          foreach ($result as $data) {
            $style_name = isset($node->album['view_imagesize']) ? $node->album['view_imagesize'] : $style_name;
            $style = \Drupal\image\Entity\ImageStyle::load($style_name);
            $file_url = $style->buildUrl($data->uri);
            $image = NULL;
            if ($i == 0) {
              $image = $cover;
            }
            $album[] = array(
              '#theme' => 'photos_image_colorbox_link',
              '#image' => $image,
              '#image_title' => $data->title,
              '#image_url' => $file_url,
              '#nid' => $node->id()
            );
            ++$i;
          }
        }
      break;
    }
    return $album;
  }

  /**
   * Get album cover.
   */
  public function getCover($fid = NULL, $title = '') {
    $pid = $this->pid;
    $cover = array();
    if (!$fid) {
      // Check album for cover fid.
      $fid = db_query("SELECT fid FROM {photos_album} WHERE pid = :pid", array(':pid' => $pid))->fetchField();
      if (!$fid) {
        // Cover not set, select an image from the album.
        $query = db_select('file_managed', 'f');
        $query->join('photos_image', 'p', 'p.fid = f.fid');
        $query->fields('f', array('fid'));
        $query->condition('p.pid', $pid);
        $fid = $query->execute()->fetchField();
      }
    }
    if ($fid) {
      // Load image.
      $photos_image = new PhotosImage($fid);
      $image = $photos_image->load();
      // Prepare node album cover data.
      $cover['fid'] = $fid;
      $cover['uri'] = $image->uri;
      $style_name = \Drupal::config('photos.settings')->get('photos_cover_imagesize') ?: 'thumbnail';
      $image->href = Url::fromUri('base:photos/album/' . $pid)->toString();
      // Set alt and title to album node title.
      $image->alt = $title;
      $image->title = $title;
      $cover_view = array(
        '#theme' => 'photos_image_html',
        '#image' => $image,
        '#style_name' => $style_name
      );
      $cover['view'] = $cover_view;
    }
    else {
      // No images in this album.
      // @todo allow admin to set default image?
    }
    return $cover;
  }

  /**
   * Set album cover.
   */
  public function setCover($fid = 0) {
    $pid = $this->pid;
    // Update cover.
    db_query('UPDATE {photos_album} SET fid = :fid WHERE pid = :pid',
      array(':fid' => $fid, ':pid' => $pid));
    // Clear node cache.
    \Drupal\Core\Cache\Cache::invalidateTags(array('node:' . $pid, 'photos:album:' . $pid));
    drupal_set_message(t('Cover successfully set.'));
  }

  /**
   * Get album images.
   */
  public function getImages($limit = 10) {
    $images = array();
    // Prepare query.
    $column = isset($_GET['field']) ? \Drupal\Component\Utility\Html::escape($_GET['field']) : '';
    $sort = isset($_GET['sort']) ? \Drupal\Component\Utility\Html::escape($_GET['sort']) : '';
    $term = \Drupal\photos\PhotosAlbum::_photos_order_value($column, $sort, $limit, array('column' => 'p.wid', 'sort' => 'asc'));
    // Query images in this album.
    $query = db_select('file_managed', 'f')
      ->extend('Drupal\Core\Database\Query\PagerSelectExtender');
    $query->join('photos_image', 'p', 'p.fid = f.fid');
    $query->join('users_field_data', 'u', 'f.uid = u.uid');
    $query->join('node_field_data', 'n', 'n.nid = p.pid');
    $query->fields('f', array('fid'));
    $query->condition('p.pid', $this->pid);
    $query->limit($term['limit']);
    $query->orderBy($term['order']['column'], $term['order']['sort']);
    $query->addTag('node_access');
    $results = $query->execute();
    // Prepare images.
    foreach ($results as $result) {
      $photos_image = new \Drupal\photos\PhotosImage($result->fid);
      $images[] = $photos_image->load();
    }
    if (isset($images[0]->fid)) {
      $node = \Drupal::entityManager()->getStorage('node')->load($this->pid);
      $images[0]->info = array(
        'pid' => $node->id(),
        'title' => $node->getTitle(),
        'uid' => $node->uid
      );
      if (isset($node->album['cover'])) {
        $images[0]->info['cover'] = $node->album['cover'];
      }
    }
    return $images;
  }

  /**
   * Limit and sort by links.
   */
  public static function _photos_order_link($arg, $count = 0, $link = 0, $limit = 0) {
    // @todo move out? This is used in recent images, user images and album.
    // @todo move count and order links out.
    // Get current path.
    $q = $current_path = \Drupal::service('path.current')->getPath();
    $field = array(
      'weight' => t('By weight'),
      'created' => t('By time'),
      'comments' => t('By comments'),
      'filesize' => t('By filesize')
    );
    // Check count image views variable.
    $photos_image_count = \Drupal::config('photos.settings')->get('photos_image_count');
    if (!$photos_image_count) {
      $field['visits'] = t('By visits');
    }
    if ($limit) {
      $query = PhotosAlbum::_photos_pager_get_query();
      $links['limit'] = '';
      if (!is_array($limit)) {
        $limit = array(5, 10, 20, 30, 40, 50);
      }
      $limit_query = $query;
      $limit_count = count($limit);
      foreach ($limit as $key => $tt) {
        $limit_query['limit'] = $tt;
        $sort = array(
          'query' => $limit_query,
          'attributes' => array(
            'class' => array(
              (isset($_GET['limit']) && $_GET['limit'] == $tt) ? 'orderac' : NULL
            ),
            'rel' => 'nofollow'
          )
        );

        $links['limit'] .= \Drupal::l($tt, Url::fromUri('base:' . $q, $sort));
        if ($limit_count <> $key) $links['limit'] .= ' ';

      }
    }
    $links['count'] = $count;
    $links['link'] = $link ? $link : NULL;
    $links['sort'] = \Drupal::l(t('Default'), Url::fromUri('base:' . $arg, array('attributes' => array('rel' => 'nofollow'))));

    $sort_links = '';
    $sort_link_count = count($field);
    foreach ($field as $key => $t) {
      if (!isset($_GET['field']) || $_GET['field'] <> $key) {
        $sort = 'desc';
        $class = 'photos_order_desc';
      }
      elseif ($_GET['sort'] == 'desc') {
        $sort = 'asc';
        $class = 'photos_order_desc orderac';
      }
      else {
        $sort = 'desc';
        $class = 'photos_order_asc orderac';
      }
      $field_query = array(
        'sort' => $sort,
        'field' => $key
      );
      if (isset($_GET['limit'])) $field_query['limit'] = \Drupal\Component\Utility\Html::escape($_GET['limit']);
      $sort_links .= \Drupal::l($t, Url::fromUri('base:' . $q, array(
        'query' => $field_query,
        'attributes' => array(
          'class' => array($class),
          'rel' => 'nofollow'
        )
      )));
      if ($key <> $sort_link_count) $sort_links .= ' ';
    }
    if ($sort_links) {
      $links['sort'] = array(
        '#markup' => $sort_links
      );
    }

    return array(
      '#theme' => 'photos_album_links',
      '#links' => $links
    );
  }

  /**
   * Returns array of query parameters.
   */
  public static function _photos_pager_get_query() {
    $query = array();
    if (!isset($string)) {
      $query = \Drupal\Component\Utility\UrlHelper::filterQueryParameters($_REQUEST, array_merge(array('limit', 'q', 'page', 'destination'), array_keys($_COOKIE)));
    }

    return $query;
  }

  /**
   * Sort order labels.
   */
  public static function _photos_order_label() {
    return array(
      'weight|asc' => t('Weight - smallest first'),
      'weight|desc' => t('Weight - largest first'),
      'created|desc' => t('Upload Date - newest first'),
      'created|asc' => t('Upload Date - oldest first'),
      'comments|desc' => t('Comments - most first'),
      'comments|asc' => t('Comments - least first'),
      'filesize|desc' => t('Filesize - smallest first'),
      'filesize|asc' => t('Filesize - largest first'),
      'visits|desc' => t('Visits - most first'),
      'visits|asc' => t('Visits - least first')
    );
  }

  /**
   * Extends photos order value.
   */
  public static function _photos_order_value_change($field, $sort) {
    // @note timestamp is deprecated, but may exist
    //  if albums are migrated from a previous version.
    $array = array(
      'weight' => 'p.wid',
      'timestamp' => 'f.fid',
      'created' => 'f.fid',
      'comments' => 'p.comcount',
      'visits' => 'p.count',
      'filesize' => 'f.filesize'
    );
    $array1 = array(
      'desc' => 'desc',
      'asc' => 'asc'
    );
    if (isset($array[$field]) && isset($array1[$sort])) {
      return array(
        'column' => $array[$field],
        'sort' => $array1[$sort]
      );
    }
    else {
      // Default if values not found.
      return array(
        'column' => 'f.fid',
        'sort' => 'desc'
      );
    }
  }

  /**
   * Query helper: sort order and limit.
   */
  public static function _photos_order_value($field, $sort, $limit, $default = 0) {
    // @todo update default to check album default!
    if (!$field && !$sort) {
      $t['order'] = !$default ? array('column' => 'f.fid', 'sort' => 'desc') : $default;
    }
    else {
      if (!$t['order'] = \Drupal\photos\PhotosAlbum::_photos_order_value_change($field, $sort)) {
        $t['order'] = !$default ? array('column' => 'f.fid', 'sort' => 'desc') : $default;
      }
    }
    if ($limit) {
      if (isset($_GET['limit']) && !$show = intval($_GET['limit'])) {
        if ($_GET['destination']) {
          $str = $_GET['destination'];
          if (preg_match('/.*limit=(\d*).*/i', $str, $mat)) $show = intval($mat[1]);
        }
      }
      $t['limit'] = isset($show) ? $show : $limit;
    }

    return $t;
  }

  /**
   * Return number of albums or photos.
   */
  public static function photos_get_count($type, $id = 0) {
    switch ($type) {
      case 'user_album':
      case 'user_image':
      case 'site_album':
      case 'site_image':
      case 'node_node':
        return db_query("SELECT value FROM {photos_count} WHERE cid = :cid AND type = :type", array(':cid' => $id, ':type' => $type))->fetchField();
      case 'node_album':
        return db_query("SELECT count FROM {photos_album} WHERE pid = :pid", array(':pid' => $id))->fetchField();
    }
  }

  /**
   * Update count.
   */
  public static function _photos_res_count($cron = 0) {
    PhotosAlbum::photos_set_count('site_album');
    PhotosAlbum::photos_set_count('site_image');
    $time = $cron ? 7200 : 0;
    // @todo optimize. Check if new images since last count.
    $cron_last = \Drupal::state()->get('system.cron_last', 0);
    if ((REQUEST_TIME - $cron_last) > $time) {
      $result = db_query('SELECT uid FROM {users} WHERE uid <> 0');
      foreach ($result as $t) {
        PhotosAlbum::photos_set_count('user_image', $t->uid);
        PhotosAlbum::photos_set_count('user_album', $t->uid);
      }
      $result = db_query('SELECT pid FROM {photos_album}');
      foreach ($result as $t) {
        PhotosAlbum::photos_set_count('node_album', $t->pid);
      }
      $result = db_query('SELECT DISTINCT(nid) FROM {photos_node}');
      foreach ($result as $t) {
        PhotosAlbum::photos_set_count('node_node', $t->nid);
      }
    }
  }

  /**
   * Calculate number of $type.
   */
  public static function photos_set_count($type, $id = 0) {
    switch ($type) {
      case 'user_image':
        $count = db_query('SELECT count(p.fid) FROM {photos_image} p INNER JOIN {file_managed} f ON p.fid = f.fid WHERE f.uid = :id',
          array(':id' => $id))->fetchField();
        $query = db_update('photos_count');
        $query->fields(array(
          'value' => $count,
          'changed' => REQUEST_TIME
        ));
        $query->condition('cid', $id);
        $query->condition('type', $type);
        $affected_rows = $query->execute();
        if (!$affected_rows) {
          db_insert('photos_count')
            ->fields(array(
              'cid' => $id,
              'changed' => REQUEST_TIME,
              'type' => $type,
              'value' => $count
            ))
            ->execute();
        }
      break;
      case 'user_album':
        $count = db_query('SELECT count(p.pid) FROM {photos_album} p INNER JOIN {node_field_data} n ON p.pid = n.nid WHERE n.uid = :uid',
          array(':uid' => $id))->fetchField();
        $query = db_update('photos_count')
          ->fields(array(
            'value' => $count,
            'changed' => REQUEST_TIME
          ))
          ->condition('cid', $id)
          ->condition('type', $type);
        $affected_rows = $query->execute();
        if (!$affected_rows) {
          db_insert('photos_count')
            ->fields(array(
              'cid' => $id,
              'changed' => REQUEST_TIME,
              'type' => $type,
              'value' => $count
            ))
            ->execute();
        }
      break;
      case 'site_album':
        $count = db_query('SELECT COUNT(pid) FROM {photos_album}')->fetchField();
        $query = db_update('photos_count')
          ->fields(array(
            'value' => $count,
            'changed' => REQUEST_TIME
          ))
          ->condition('cid', 0)
          ->condition('type', $type);
        $affected_rows = $query->execute();
        if (!$affected_rows) {
          db_insert('photos_count')
            ->fields(array(
              'cid' => 0,
              'changed' => REQUEST_TIME,
              'type' => $type,
              'value' => $count
            ))
            ->execute();
        }
      break;
      case 'site_image':
        $count = db_query('SELECT COUNT(fid) FROM {photos_image}')->fetchField();
        $query = db_update('photos_count')
          ->fields(array(
            'value' => $count,
            'changed' => REQUEST_TIME
          ))
          ->condition('cid', 0)
          ->condition('type', $type);
        $affected_rows = $query->execute();
        if (!$affected_rows) {
          db_insert('photos_count')
            ->fields(array(
              'cid' => 0,
              'changed' => REQUEST_TIME,
              'type' => $type,
              'value' => $count
            ))
            ->execute();
        }
      break;
      case 'node_node':
        $count = db_query('SELECT COUNT(nid) FROM {photos_node} WHERE nid = :nid', array(':nid' => $id))->fetchField();
        $query = db_update('photos_count')
          ->fields(array(
            'value' => $count,
            'changed' => REQUEST_TIME
          ))
          ->condition('cid', $id)
          ->condition('type', $type);
        $affected_rows = $query->execute();
        if (!$affected_rows) {
          db_insert('photos_count')
            ->fields(array(
              'cid' => $id,
              'changed' => REQUEST_TIME,
              'type' => $type,
              'value' => $count
            ))
            ->execute();
        }
      break;
      case 'node_album':
        db_query("UPDATE {photos_album} SET count = (SELECT COUNT(fid) FROM {photos_image} WHERE pid = :pid) WHERE pid = :pid", array(':pid' => $id));
      break;
    }
  }

  /**
   * Tracks number of albums created and number of albums allowed.
   */
  public static function photos_user_count() {
    $user = \Drupal::currentUser();
    $array = $user->getRoles();
    $t['create'] = PhotosAlbum::photos_get_count('user_album', $user->id());
    // @todo check all roles? See what number is highest.
    // @todo upgrade path? Check D7 role id and convert pnum variables as needed.
    $t['total'] = \Drupal::config('photos.settings')->get('photos_pnum_' . $array[0]) ? \Drupal::config('photos.settings')->get('photos_pnum_' . $array[0]) : 20;

    $t['remain'] = ($t['total'] - $t['create']);
    if ($user->id() <> 1 && $t['remain'] <= 0) {
      $t['rest'] = 1;
    }
    return $t;
  }

  /**
   * User albums.
   */
  public static function _photos_useralbum_option($uid = 0, $current = 0) {
    if (!$uid) $uid = \Drupal::currentUser()->id();
    $output = array();

    // @todo cleanup and test.
    $result = db_query('SELECT n.nid, n.title FROM {node_field_data} n
      INNER JOIN {photos_album} a
      ON a.pid = n.nid
      WHERE n.uid = :uid
      ORDER BY n.nid DESC', array(':uid' => $uid));
    $query = db_select('node_field_data', 'n');
    $query->join('photos_album', 'a', 'a.pid = n.nid');
    $query->fields('n', array('nid', 'title'));
    $query->condition('n.uid', $uid);
    $query->orderBy('n.nid', 'DESC');
    $result = $query->execute();

    $true = FALSE;
    foreach ($result as $a) {
      $choice = new \stdClass();
      $choice->option = array($a->nid => $a->title);
      $output[] = $choice;
      $true = TRUE;
    }
    if ($current) {
      $choice = new \stdClass();
      $choice->option = array($current[0] => $current[1]);
      $output[] = $choice;
    }
    if (!$true) {
      $output = array(t('You do not have an album yet.'));
    }

    return $output;
  }

  /**
   * Handles JSON for slideshow plugin(s).
   *
   * @note not currently in use.
   */
  public static function _photos_json($type, $v) {
    if ($type == 'en') {
      return \Drupal\Component\Serialization\Json::encode($v);
    }
    else {
      return \Drupal\Component\Serialization\Json::decode($v);
    }
  }

}
