<?php

namespace Drupal\photos;


class PhotosUpload {

  /**
   * Rename file with random name.
   */
  public static function _photos_rename($name = 0, $ext = 0) {
    // @todo use transliterate? Or add more options.
    if (\Drupal::config('photos.settings')->get('photos_rname')) {
      if ($name) {
        $name_parts = explode('.', $name);
        return round(rand(15770, 967049700)) . REQUEST_TIME . '.' . ($ext ? $ext : end($name_parts));
      }
      if (!empty($_FILES['files'])) {
        foreach ($_FILES['files']['name'] as $field => $filename) {
          $filename_parts = explode('.', $filename);
          $_FILES['files']['name'][$field] = round(rand(15770, 967049700)) . REQUEST_TIME . '.' . ($ext ? $ext : end($filename_parts));
        }
      }
    }
    elseif ($name) {
      return $name;
    }
  }

  /**
   * Write files.
   */
  public static function _photos_save_data($file, $val = array()) {
    $errors = array();
    foreach ($val as $function => $args) {
      array_unshift($args, $file);
      $errors = array_merge($errors, call_user_func_array($function, $args));
    }
    if (!empty($errors)) {
      $message = t('The selected file %name could not be uploaded.', array('%name' => $file->filename));
      if (count($errors) > 1) {
        $message .= '<ul><li>' . implode('</li><li>', $errors) . '</li></ul>';
      }
      else {
        $message .= ' ' . array_pop($errors);
      }
      drupal_set_message($message);
      return 0;
    }
    $file->save();

    return $file->id();
  }

  /**
   * Image written to database.
   */
  public static function photos_image_save($file) {
    // @todo re-write.
    // @todo check title, description, weight.
    //  - maybe pass file object and array of other vars.
    $exif = ($file->getMimeType() == 'image/jpeg') ? 1 : 0;
    if ($file->id() && isset($file->pid)) {
      $fid = $file->id();
      $pid = $file->pid;
      db_merge('photos_image')
        ->key(array(
          'fid' => $file->id()
        ))
        ->fields(array(
          'pid' => $file->pid,
          'title' => isset($file->title) ? $file->title : $file->getFilename(),
          'des' => isset($file->des) ? $file->des : '',
          'wid' => isset($file->wid) ? $file->wid : 0,
          'comcount' => 0,
          'count' => 0,
          'exif' => $exif
        ))
        ->execute();
      if (isset($fid) && !empty($fid)) {
        if (isset($file->nid)) {
          db_insert('photos_node')
            ->fields(array(
              'nid' => $file->nid,
              'fid' => $file->id()
            ))
            ->execute();
        }
        if (\Drupal::config('photos.settings')->get('photos_user_count_cron')) {
          $user = \Drupal::currentUser();
          \Drupal\photos\PhotosAlbum::photos_set_count('user_image', ($file->getOwnerId ? $file->getOwnerId : $user->id()));
          \Drupal\photos\PhotosAlbum::photos_set_count('node_album', $file->pid);
          isset($file->nid) ? \Drupal\photos\PhotosAlbum::photos_set_count('node_node', $file->nid) : NULL;
        }
        // Save file and add file usage.
        $file_usage = \Drupal::service('file.usage');
        $file_usage->add($file, 'photos', 'node', $pid);
        // Check admin setting for maximum image resolution.
        if ($photos_size_max = \Drupal::config('photos.settings')->get('photos_size_max')) {
          // Will scale image if needed.
          file_validate_image_resolution($file, $photos_size_max);
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Prepare photos custom directory structure.
   */
  public static function photos_check_path($type = 'default', $file = FALSE, $account = FALSE) {
    if (!$account) {
      $user = \Drupal::currentUser();
      $account = $user;
    }
    $path = array();
    if (\Drupal::config('photos.settings')->get('photos_path')) {
      $mm = \Drupal::service('date.formatter')->format(REQUEST_TIME, 'custom', "Y|m|d");
      $m = explode('|', $mm);
      $a = array('%uid' => $account->id(), '%username' => $account->getUsername(), '%Y' => $m[0], '%m' => $m[1], '%d' => $m[2]);
      $b = strtr(\Drupal::config('photos.settings')->get('photos_path'), $a);
      $path = explode('/', $b);
    }
    else {
      $path[] = 'photos';
    }
    switch ($type) {
      case 'default':
        $scheme = file_default_scheme();
      break;
      case 'private':
        $scheme = 'private';
      break;
      case 'public':
        $scheme = 'public';
      break;
    }
    $dirs = array();
    foreach ($path as $folder) {
      $dirs[] = $folder;
      $t = $scheme . '://' . implode('/', $dirs);
      if (!file_prepare_directory($t, FILE_CREATE_DIRECTORY)) {
        return FALSE;
      }
    }

    return $t;
  }

  /**
   * Unzip archive of image files.
   */
  public static function _photos_unzip($source, $value, $scheme = 'default', $account = FALSE) {
    $user = \Drupal::currentUser();
    $file_count = 0;
    if (version_compare(PHP_VERSION, '5') >= 0) {
      if (!is_file($source)) {
        drupal_set_message(t('Compressed file does not exist, please check the path: @src', array('@src' => $source)));
        return 0;
      }
      $type = array('jpg', 'gif', 'png', 'jpeg', 'JPG', 'GIF', 'PNG', 'JPEG');
      $zip = new \ZipArchive();
      // Get relative path.
      $relative_path = \Drupal::service('file_system')->realpath(file_default_scheme() . "://") . '/';
      $source = str_replace(file_default_scheme() . '://', $relative_path, $source);
      // Open zip archive.
      if ($zip->open($source) === TRUE) {
        for ($x = 0; $x < $zip->numFiles; ++$x) {
          $image = $zip->statIndex($x);
          $filename_parts = explode('.', $image['name']);
          $ext = end($filename_parts);
          if (in_array($ext, $type)) {
            $path = file_create_filename(PhotosUpload::_photos_rename($image['name'], $ext), PhotosUpload::photos_check_path($scheme, '', $account));
            if ($temp_file = file_save_data($zip->getFromIndex($x), $path)) {
              // Get image info.
              $image_info = \Drupal::service('image.factory')->get($temp_file->uri);
              // Update file values.
              $temp_file->pid = $value->pid;
              $temp_file->nid = $value->nid;
              // Use image file name as title.
              $temp_file->title = $image['name'];
              $temp_file->des = $value->des;
              // Prepare file entity.
              $file = $temp_file;
              // Save image.
              if ($file_fid = \Drupal\photos\PhotosUpload::_photos_save_data($file)) {
                if (PhotosUpload::photos_image_save($file)) $file_count++;
              }
            }
          }
        }
        $zip->close();
        // Delete zip file.
        file_unmanaged_delete($source);
      }
      else {
        drupal_set_message(t('Compressed file does not exist, please try again: @src', array('@src' => $source)), 'warning');
      }
    }

    return $file_count;
  }

}
