<?php

namespace Drupal\photos\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;

/**
 * Provides a 'Photo information' block.
 *
 * @Block(
 *   id = "photos_information",
 *   admin_label = @Translation("Photo Information"),
 *   category = @Translation("Photos")
 * )
 */
class PhotosInformation extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = false) {
    // Check if user can view photos.
    if (\Drupal::currentUser()->hasPermission('view photo')) {
      $access = AccessResult::allowed();
    }
    else {
      $access = AccessResult::forbidden();
    }
    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $content = array();
    // Retrieve existing configuration for this block.
    $config = $this->getConfiguration();

    // Check which pager to load.
    $fid = \Drupal::routeMatch()->getParameter('file');
    $current_path = \Drupal::service('path.current')->getPath();
    $arg = explode('/', $current_path);
    $pager_type = 'pid';
    if (isset($arg[3])) {
      if ($arg[1] == 'photos' && $arg[2] == 'image' && is_numeric($arg[3]) && isset($_GET['photos_sub'])) {
        $fid = $arg[3];
        $pager_type = 'sub';
        $pager_id = (int)$_GET['photos_sub'];
      }
      elseif ($arg[1] == 'photos' && $arg[2] == 'image' && is_numeric($arg[3])) {
        $fid = $arg[3];
        $pager_type = 'pid';
      }
      elseif (isset($arg[5]) && $arg[1] == 'photos' && $arg[2] == 'user' && is_numeric($arg[3]) && is_numeric($arg[5])) {
        $fid = $arg[5];
        $uid = $arg[3];
        $pager_id = $arg[3];
        $pager_type = 'uid';
      }
    }

    if (isset($fid)) {
      // Get current image.
      $query = db_select('photos_image', 'p');
      $query->join('file_managed', 'f', 'f.fid = p.fid');
      $query->join('node_field_data', 'n', 'n.nid = p.pid');
      $query->join('users_field_data', 'u', 'f.uid = u.uid');
      $query->fields('p', array('count', 'comcount', 'exif', 'des'))
        ->fields('f', array('uri', 'created', 'filemime', 'fid'))
        ->fields('n', array('nid', 'title'))
        ->fields('u', array('name', 'uid'))
        ->condition('p.fid', $fid);
      $query->addTag('node_access');
      $image = $query->execute()->fetchObject();
      if ($image) {
        if ($pager_type == 'pid') {
          $pager_id = $image->nid;
        }
        // Get pager image(s).
        $photos_image = new \Drupal\photos\PhotosImage($fid);
        $image->pager = $photos_image->pager($pager_id, $pager_type);
        if (\Drupal::config('photos.settings')->get('photos_slide') && \Drupal::moduleHandler()->moduleExists('dfgallery')) {
          $image->slide_url = Url::fromUri('base:photos/album/' . $image->nid . '/slide')->toString();
        }

        $content = array(
          '#theme' => 'photos_image_block',
          '#image' => $image,
          '#attached' => array(
            'library' => 'photos/photos.block.information'
          ),
          '#cache' => array(
            'max_age' => 0
          )
        );
      }
      return $content;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

}
